@@ -23,21 +23,18 @@
 #define SQUASH_DATABASE_H
 
 #include <dirent.h> /* For readdir(), etc. */
-#include <pthread.h>
 
 /* Initial database allocation size */
 #define INITIAL_DB_SIZE 2500
 
-/* File Extensions to check, values defined at top of database.c */
-extern const db_extension_info_t db_extensions[];
-extern const int db_extensions_size;
-
 /*
  * Prototypes
  */
 void *setup_database( void *data );
 
-void load_db( void );
+void load_db_filenames( void );
+void load_masterlist( void );
+void save_masterlist( void );
 void clear_song_meta( song_info_t *song );
 void clear_db( void );
 
@@ -45,7 +42,8 @@ meta_key_t *get_meta_data( song_info_t *song_info, char *meta_key );
 song_info_t *find_song_by_filename( char *filename );
 db_search_result_t find_matches( char *key, char *keyword );
 
-void load_meta_data( song_info_t *song );
+void load_meta_data( song_info_t *song, enum meta_type_e which );
+void load_all_meta_data( enum meta_type_e which );
 void save_song( song_info_t *song );
 
 void insert_meta_data( void *data, char *header, char *key, char *value );
@@ -56,7 +54,7 @@ void set_stat_data( void *data, char *header, char *key, char *value );
 void save_stat_data( song_info_t *song, FILE *file );
 bool is_stat_data_changed( song_info_t *song );
 
-void _load_file( char *filename );
-void _walk_filesystem( char *base_dir, void(*loader)(char *) );
+void _load_file( char *filename, bool trust );
+void _walk_filesystem( char *base_dir, void(*loader)(char *, bool) );
 
 #endif
