@@ -22,6 +22,9 @@
 #ifndef SQUASH_PLAYER_H
 #define SQUASH_PLAYER_H
 
+#define PLAYER_MAX_BUFFER_SIZE 2048
+#define PLAYER_MAX_BUFFER_PCM_SIZE (1024*1024)
+
 /*
  * Global Data
  */
@@ -30,11 +33,12 @@ extern song_functions_t song_functions[];
 /*
  * Prototypes
  */
+void *frame_decoder( void *input_data );
 void *player( void *input_data );
-song_info_t *get_next_song_info( void );
+void get_next_song_info( song_info_t **song, long *start_position );
 void done_with_song_info( song_info_t *song );
 int detect_silence( frame_data_t frame_data, unsigned int *silence_duration );
-void set_now_playing_info( song_info_t *song, long duration );
+void set_now_playing_info( song_info_t *song, long start_position );
 double *get_spectrum(char *pcm_data, int pcm_length);
 void player_queue_command( enum player_command_e command );
 #endif
