@@ -24,6 +24,7 @@
 #include "global.h"
 #include "database.h"
 #include "display.h"
+#include "sound.h" /* for sound_set_volume() */
 
 /* File Extensions to use. */
 const db_extension_info_t db_extensions[] = {
@@ -39,24 +40,193 @@ config_keys_t config_keys[ CONFIG_KEY_COUNT ] = {
     { "Database", "Info_Path", (void *)&config.db_paths[ BASENAME_META ], TYPE_STRING },
     { "Database", "Stat_Path", (void *)&config.db_paths[ BASENAME_STAT ], TYPE_STRING },
     { "Database", "Song_Path", (void *)&config.db_paths[ BASENAME_SONG ], TYPE_STRING },
+    { "Database", "Masterlist_Filename", (void *)&config.db_masterlist_path, TYPE_STRING },
     { "Database", "Readonly", (void *)&config.db_readonly, TYPE_INT },
     { "Database", "Save_Info", (void *)&config.db_saveinfo, TYPE_INT },
     { "Database", "Overwrite_Info", (void *)&config.db_overwriteinfo, TYPE_INT },
+    { "Database", "Manual_Rating_Bias", (void *)&config.db_manual_rating_bias, TYPE_DOUBLE },
+    { "Global", "State_Filename", (void *)&config.global_state_path, TYPE_STRING },
     { "Global", "Control_Filename", (void *)&config.input_fifo_path, TYPE_STRING },
 #ifdef DEBUG
     { "Global", "Log_Filename", (void *)&config.squash_log_path, TYPE_STRING },
 #endif
-    { "Playlist", "Filename", (void *)&config.playlist_manager_playlist_path, TYPE_STRING },
+#ifdef EMPEG_DSP
+    { "Empeg", "save_volume_minimum", (void *)&config.min_save_volume, TYPE_INT },
+    { "Empeg", "save_volume_maximum", (void *)&config.max_save_volume, TYPE_INT },
+#endif
     { "Playlist", "Size", (void *)&config.playlist_manager_playlist_size, TYPE_INT },
     { "Pastlist", "Size", (void *)&config.playlist_manager_pastlist_size, TYPE_INT }
 };
 
+#ifdef EMPEG
+/*
+ * Setup empeg menus
+ */
+menu_list_t menu_list[ EMPEG_SCREEN_COUNT ] = (menu_list_t []) {
+    { (menu_item_t []){
+            (menu_item_t) { "Main Menu",            0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Currently Playing",    1, EMPEG_SCREEN_PLAY, TRUE },
+            (menu_item_t) { "Upcoming Songs",       1, EMPEG_SCREEN_PLAYLIST, TRUE },
+            (menu_item_t) { "Statistics",           1, EMPEG_SCREEN_GLOBAL_SONG_INFO, TRUE },
+            (menu_item_t) { "Quit Squash",          1, EMPEG_SCREEN_QUIT, TRUE },
+            (menu_item_t) { "Change Settings",      1, EMPEG_SCREEN_SETTINGS, FALSE }
+        },
+        6, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Currently Playing",    0, EMPEG_SCREEN_PLAY, TRUE },
+            (menu_item_t) { "Rate This Song",       1, EMPEG_SCREEN_PLAY_RATE_SONG, TRUE },
+            (menu_item_t) { "Song Info",            1, EMPEG_SCREEN_PLAY_SONG_INFO, TRUE }
+        },
+        4, 1
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_PLAY, FALSE },
+            (menu_item_t) { "Song Details",         0, EMPEG_SCREEN_PLAY_SONG_INFO, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_PLAY, FALSE },
+            (menu_item_t) { "Rate This Song",       0, EMPEG_SCREEN_PLAY_RATE_SONG, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Upcoming Songs",       0, EMPEG_SCREEN_PLAYLIST, TRUE },
+            (menu_item_t) { "Rate This Song",       1, EMPEG_SCREEN_PLAYLIST_RATE_SONG, TRUE },
+            (menu_item_t) { "Song Info",            1, EMPEG_SCREEN_PLAYLIST_SONG_INFO, TRUE },
+            (menu_item_t) { "Remove from list",     1, EMPEG_SCREEN_PLAYLIST, TRUE }
+        },
+        5, 1
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_PLAYLIST, FALSE },
+            (menu_item_t) { "Song Info",            0, EMPEG_SCREEN_PLAYLIST_SONG_INFO, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_PLAYLIST, FALSE },
+            (menu_item_t) { "Rate This Song",       0, EMPEG_SCREEN_PLAYLIST_RATE_SONG, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Statistics",           0, EMPEG_SCREEN_GLOBAL_SONG_INFO, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Quit Squash",          0, EMPEG_SCREEN_QUIT, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Change Settings",      0, EMPEG_SCREEN_SETTINGS, FALSE },
+            (menu_item_t) { "Set Rating Bias",      1, EMPEG_SCREEN_SET_RATING_BIAS, TRUE },
+            (menu_item_t) { "Display Brightness",   1, EMPEG_SCREEN_SET_DISPLAY_BRIGHTNESS, TRUE }
+        },
+        4, 1
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Display Brightness",   0, EMPEG_SCREEN_SET_DISPLAY_BRIGHTNESS, TRUE }
+        },
+        2, 0
+    },
+    { (menu_item_t []){
+            (menu_item_t) { "Go Back",              0, EMPEG_SCREEN_MAIN_MENU, FALSE },
+            (menu_item_t) { "Set Rating Bias",      0, EMPEG_SCREEN_SET_RATING_BIAS, TRUE }
+        },
+        2, 0
+    }
+};
+#endif
+
+void save_state() {
+    song_info_t *cur_song;
+    long cur_pos;
+    FILE *state_file;
+    time_t current_time;
+    song_queue_entry_t *cur_song_queue_entry;
+
+    if( config.db_readonly ) {
+        return;
+    }
+
+    /* Open the file to write out the state */
+    if( (state_file = fopen(config.global_state_path, "w")) == NULL ) {
+        squash_error( "Can't open file \"%s\" for writing", config.global_state_path );
+    }
+
+    /* Write out header */
+    time( &current_time );
+    fprintf( state_file, "# Saved Squash state\n" );
+    fprintf( state_file, "# Auto-Generated on %s\n", ctime(&current_time) );
+
+    /* Write out the state */
+    cur_song = player_info.song;
+    cur_pos = player_info.current_position;
+    fprintf(state_file, "current_entry=0\n");
+    if( cur_song ) {
+        fprintf(state_file, "entry_filename=%s/%s\n", cur_song->basename[ BASENAME_SONG ], cur_song->filename);
+        fprintf(state_file, "entry_position=%ld\n", cur_pos);
+    }
+
+    for( cur_song_queue_entry = song_queue.head; cur_song_queue_entry != NULL; cur_song_queue_entry = cur_song_queue_entry->next ) {
+        cur_song = cur_song_queue_entry->song_info;
+        cur_pos = cur_song_queue_entry->start_position;
+        fprintf(state_file, "entry_filename=%s/%s\n", cur_song->basename[ BASENAME_SONG ], cur_song->filename);
+        fprintf(state_file, "entry_position=%ld\n", cur_pos);
+    }
+
+    while( state_info.current_song < state_info.raw_song_count ) {
+        fprintf(state_file, "entry_filename=%s\n", state_info.raw_songs[state_info.current_song].filename );
+        fprintf(state_file, "entry_position=%ld\n", state_info.raw_songs[state_info.current_song].position );
+        state_info.current_song++;
+    }
+
+#ifdef EMPEG_DSP
+    if( player_info.device ) {
+        int volume = player_info.device->volume[0];
+        if( volume > config.max_save_volume ) {
+            volume = config.max_save_volume;
+        } else if( volume < config.min_save_volume ) {
+            volume = config.min_save_volume;
+        }
+        fprintf(state_file, "volume=%d\n", volume);
+    }
+#endif
+
+#ifdef EMPEG
+    fprintf( state_file, "brightness=%d\n", display_info.brightness );
+#endif
+
+#ifndef NO_NCURSES
+    {
+        int i;
+        for( i = 0; i < WIN_COUNT; i++ ) {
+            fprintf(state_file, "window_number=%d\n", i);
+            fprintf(state_file, "window_state=%d\n", display_info.window[i].state);
+        }
+    }
+#endif
+
+    /* Close the state file */
+    fclose( state_file );
+}
 
 /*
  * Return the file format of the song (mp3, ogg, etc.).
  * TODO: Check using something smarter than just the filename
  */
-enum song_type_e get_song_type( const char *file_name ) {
+enum song_type_e get_song_type( const char *base_name, const char *file_name ) {
     enum song_type_e type;
     unsigned int file_name_length;
 
@@ -71,10 +241,44 @@ enum song_type_e get_song_type( const char *file_name ) {
     /* Determine the correct play functions */
     if( !strncasecmp(file_name + file_name_length - 4, ".mp3", 4) ) {
         type = TYPE_MP3;
-    } else  if( !strncasecmp(file_name + file_name_length - 4, ".ogg", 4) ) {
+    } else if( !strncasecmp(file_name + file_name_length - 4, ".ogg", 4) ) {
         type = TYPE_OGG;
+    } else if( !strncasecmp(file_name + file_name_length - 5, ".flac", 5) ) {
+        type = TYPE_FLAC;
     }
 
+#ifdef USE_MAGIC
+    if( type == TYPE_UNKNOWN ) {
+        char buf[4] = {'\0','\0','\0','\0'};
+        FILE *file;
+        char *full_name;
+        if( base_name == NULL ) {
+            full_name = strdup( file_name );
+        } else {
+            asprintf( &full_name, "%s/%s", base_name, file_name );
+        }
+        if( (file = fopen(full_name, "r")) == NULL ) {
+            squash_error( "Can't open %s to determine filetype", file_name );
+        }
+        fread( buf, 4, 1, file );
+        if( (buf[0] == '\xFF' && (buf[1] & '\xE0') == '\xE0' ) ||
+            (buf[0] == 'I' && buf[1] == 'D' && buf[2] == '3' && buf[3] == 3) ) {
+            squash_log("Magically mp3-delicious");
+            type = TYPE_MP3;
+        } else if ( buf[0] == 'O' && buf[1] == 'g' && buf[2] == 'g' && buf[3] == 'S' ) {
+            squash_log("Magically ogg-delicious");
+            type = TYPE_OGG;
+        } else if ( buf[0] == 'f' && buf[1] == 'L' && buf[2] == 'a' && buf[3] == 'C' ) {
+            squash_log("Magically flac-delicious");
+            type = TYPE_FLAC;
+        } else {
+            squash_log("Just ain't enough magic in the world");
+        }
+        fclose( file );
+        squash_free( full_name );
+    }
+#endif
+
     /* Return the information */
     return type;
 }
@@ -426,6 +630,10 @@ void set_config( void *data, char *header, char *key, char *value ) {
  * Expands a path name (tilde expansion, etc.).
  */
 void expand_path( char **path ) {
+#ifdef EMPEG
+    /* This routine does a nice segfault on the empeg unit. */
+    return;
+#else
     wordexp_t wordexp_buffer;
     if( *path == NULL ) {
         return;
@@ -445,6 +653,7 @@ void expand_path( char **path ) {
     squash_free( *path );
     *path = strdup( wordexp_buffer.we_wordv[0] );
     wordfree( &wordexp_buffer );
+#endif
 }
 
 /* Reads configuration values */
@@ -455,15 +664,22 @@ void init_config( void ) {
     config.db_paths[ BASENAME_SONG ] = strdup(".");
     config.db_paths[ BASENAME_META ] = NULL;
     config.db_paths[ BASENAME_STAT ] = NULL;
+    config.db_masterlist_path = NULL;
     config.db_readonly = 0;
     config.db_saveinfo = 1;
     config.db_overwriteinfo = 0;
+    config.db_manual_rating_bias = 0.5;
 
     /* Control Options */
+    config.global_state_path = strdup("~/.squash_state");
     config.input_fifo_path = strdup("~/.squash_control");
 
+#ifdef EMPEG_DSP
+    config.min_save_volume = 40;
+    config.max_save_volume = 70;
+#endif
+
     /* Playlist Manager Options */
-    config.playlist_manager_playlist_path = strdup("~/.squash_playlist");
     config.playlist_manager_playlist_size = 32;
     config.playlist_manager_pastlist_size = 32;
 
@@ -474,14 +690,19 @@ void init_config( void ) {
 
     /* Load Config files */
 
+#ifdef EMPEG
+    /* Load global squash.conf */
+    parse_file( "/drive0/squash/squash.conf", set_config, NULL );
+#else
     /* Load global squash.conf */
     parse_file( "/etc/squash.conf", set_config, NULL );
 
     /* Load user squash.conf */
-    config_path = "~/.squash";
+    config_path = strdup("~/.squash");
     expand_path( &config_path );
     parse_file( config_path, set_config, NULL );
     free( config_path );
+#endif
 
     /* Load environment specified squash.conf */
     if( (config_path = getenv("SQUASH_CONF")) != NULL ) {
@@ -489,15 +710,25 @@ void init_config( void ) {
     }
 
     /* Expand Paths */
+    expand_path( &config.global_state_path );
     expand_path( &config.input_fifo_path );
-    expand_path( &config.playlist_manager_playlist_path );
     expand_path( &config.db_paths[ BASENAME_SONG ] );
     expand_path( &config.db_paths[ BASENAME_META ] );
     expand_path( &config.db_paths[ BASENAME_STAT ] );
+    expand_path( &config.db_masterlist_path );
 #ifdef DEBUG
     expand_path( &config.squash_log_path );
 #endif
 
+#ifdef EMPEG_DSP
+    if( config.min_save_volume < 0 ) {
+        config.min_save_volume = 0;
+    }
+    if( config.max_save_volume > 100 ) {
+        config.max_save_volume = 100;
+    }
+#endif
+
     /* Update any remaining defaults */
     if( config.db_paths[ BASENAME_META ] == NULL ) {
         config.db_paths[ BASENAME_META ] = strdup( config.db_paths[ BASENAME_SONG ] );
