@@ -28,8 +28,14 @@
  * The value will be between -1 and 1 exclusive.
  */
 double get_rating( stat_info_t stat ) {
-    return (double)( stat.play_count - stat.skip_count ) /
-           (double)( stat.play_count + stat.skip_count + 1 );
+    double auto_rating = (double)( stat.play_count - stat.skip_count ) /
+                         (double)( stat.play_count + stat.skip_count + 1 );
+    if( stat.manual_rating == -1 ) {
+        return auto_rating;
+    } else {
+        return auto_rating * (1 - config.db_manual_rating_bias ) +
+               ((double)stat.manual_rating-5.0)/5.0 * config.db_manual_rating_bias;
+    }
 }
 
 /*
@@ -41,28 +47,35 @@ double get_rating( stat_info_t stat ) {
  */
 void start_song_picker() {
     int i;
-    double sum = 0.0;
-    double sqr_sum = 0.0;
-    long play_sum = 0.0;
-    long play_sqr_sum = 0.0;
-    long skip_sum = 0.0;
-    long skip_sqr_sum = 0.0;
+
+    squash_wlock( database_info.lock );
+
+    database_info.sum = 0;
+    database_info.sqr_sum = 0;
+    database_info.play_sum = 0;
+    database_info.play_sqr_sum = 0;
+    database_info.skip_sum = 0;
+    database_info.skip_sqr_sum = 0;
+
     for( i = 0; i < database_info.song_count; i++ ) {
-        double rating = get_rating( database_info.songs[i].stat );
-        sum += rating;
-        sqr_sum += rating * rating;
-        play_sum += database_info.songs[i].stat.play_count;
-        play_sqr_sum += database_info.songs[i].stat.play_count * database_info.songs[i].stat.play_count;
-        skip_sum += database_info.songs[i].stat.skip_count;
-        skip_sqr_sum += database_info.songs[i].stat.skip_count * database_info.songs[i].stat.skip_count;
+        double rating;
+#ifdef EMPEG
+        squash_wunlock( database_info.lock );
+        sched_yield();
+        squash_wlock( database_info.lock );
+#endif
+        rating = get_rating( database_info.songs[i].stat );
+        database_info.sum += rating;
+        database_info.sqr_sum += rating * rating;
+        database_info.play_sum += database_info.songs[i].stat.play_count;
+        database_info.play_sqr_sum += database_info.songs[i].stat.play_count * database_info.songs[i].stat.play_count;
+        database_info.skip_sum += database_info.songs[i].stat.skip_count;
+        database_info.skip_sqr_sum += database_info.songs[i].stat.skip_count * database_info.songs[i].stat.skip_count;
     }
-    database_info.sum = sum;
-    database_info.sqr_sum = sqr_sum;
-    database_info.play_sum = play_sum;
-    database_info.play_sqr_sum = play_sqr_sum;
-    database_info.skip_sum = skip_sum;
-    database_info.skip_sqr_sum = skip_sqr_sum;
+
     database_info.stats_loaded = TRUE;
+
+    squash_wunlock( database_info.lock );
 }
 
 /*
@@ -70,11 +83,11 @@ void start_song_picker() {
  * deviation) to convert X to a Z value.
  */
 bool normal_test( double x, double a, double s ) {
-    double pdf[5] = {    0.0000,
-                        0.3413,
-                        0.4772,
-                        0.4987,
-                        0.5000 };
+    double pdf[5] = { 0.0000,
+                      0.3413,
+                      0.4772,
+                      0.4987,
+                      0.5000 };
     double z;
     bool sign;
     int pdf_pos;
@@ -123,8 +136,8 @@ unsigned int pick_song() {
     unsigned int canidate;
     double canidate_rating;
 
-    /* This is a random pick.  It sucks.
-    return (int)((double)database_info.song_count * rand() / (RAND_MAX + 1.0)); */
+    /* This is a random pick.  It sucks. */
+    /* return (int)((double)database_info.song_count * rand() / (RAND_MAX + 1.0)); */
 
     avg = database_info.sum / database_info.song_count;
     std_dev = sqrt( fabs(database_info.sqr_sum / database_info.song_count - avg*avg) );
